-----------------------------------------------------------------------------------------------
-- Client Lua Script for CircleShortcuts
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- CircleShortcuts Module Definition
-----------------------------------------------------------------------------------------------
local ChatLog
local Circles

local CircleShortcuts = {}
local CS_Inst 

local sShortcut -- for setting prompt of input
 
-----------------------------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------------------------
local fnInitCircleOrig
local fnOnChatInputReturnOrig
local fnOnInputChangedOrig
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CircleShortcuts:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	self.tChannels = {}

   	return o
end

function CircleShortcuts:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- CircleShortcuts OnLoad
-----------------------------------------------------------------------------------------------
function CircleShortcuts:OnLoad()
    self.xmlDoc = XmlDoc.CreateFromFile("CircleShortcuts.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
		
	-- circles hooks
	Circles = Apollo.GetAddon("Circles")
	if Circles then
		fnInitCircleOrig = Circles.OnGenericEvent_InitializeCircles
		Circles.OnGenericEvent_InitializeCircles = self.InitCircle
		
		Circles.OnEditBoxChanged = self.OnEditBoxChanged
	end
	
	-- chatlog hooks
	ChatLog = Apollo.GetAddon("ChatLog")
	if ChatLog then
		fnOnChatInputReturnOrig = ChatLog.OnChatInputReturn
		ChatLog.OnChatInputReturn = self.OnChatInputReturn
		
		fnOnInputChangedOrig = ChatLog.OnInputChanged
		ChatLog.OnInputChanged = self.OnInputChanged
	end
	
	self:UpdateCommands()
end

function CircleShortcuts:OnDocLoaded()
	if self.xmlDoc == nil or not self.xmlDoc:IsLoaded() then return end
	
	self:UpdateCommands()
end

-- chatlog hooked functions
function CircleShortcuts:OnChatInputReturn(wndHandler, wndControl, strText)
	local strNewText = strText

	for k, v in pairs(CS_Inst.tChannels) do
		if string.sub(strText, 2, string.len(k) + 1) == k then
			strNewText = "/" .. v.sCommand .. string.sub(strText, string.len(k) + 2, string.len(strText))	
		end
	end
	
	fnOnChatInputReturnOrig(self, wndHandler, wndControl, strNewText)
	
	local wndForm = wndControl:GetParent()
	local tChatData = wndForm:GetData()
	
	local tInput = ChatSystemLib.SplitInput(strNewText)
	local channelCurrent = tInput.channelCommand or tChatData.channelCurrent
	
	local sPrompt
	for k, v in pairs(CS_Inst.tChannels) do
		if v.sCommand == channelCurrent:GetCommand() then
			sPrompt = k
		end
	end
	
	if sPrompt then
		wndControl:SetPrompt(sPrompt)
	end
end

function CircleShortcuts:OnInputChanged(wndHandler, wndControl, strText)
	local strNewText = strText

	for k, v in pairs(CS_Inst.tChannels) do
		if string.sub(strText, 2, string.len(k) + 1) == k then
			strNewText = "/" .. v.sCommand .. string.sub(strText, string.len(k) + 2, string.len(strText))	
		end
	end
	
	fnOnInputChangedOrig(self, wndHandler, wndControl, strNewText)
end

-- circles hooked functions
function CircleShortcuts:InitCircle(wndParent, guildCurr)
	fnInitCircleOrig(self, wndParent, guildCurr)
	
	local wndSearch = Apollo.LoadForm(CS_Inst.xmlDoc, "SearchBox", self.tWndRefs.wndMain, self)
	wndSearch:FindChild("EditBox"):AddEventHandler("EditBoxReturn", "OnEditBoxChanged", self)
	wndSearch:FindChild("EditBox"):SetPrompt("Chat shortuct")
	
	wndSearch:FindChild("EditBox"):GetParent():SetData(guildCurr)
	wndSearch:FindChild("EditBox"):SetData("")
	
	for k, v in pairs(CS_Inst.tChannels) do
		if v.sId == guildCurr:GetChannel():GetUniqueId() then
			wndSearch:FindChild("EditBox"):SetData(k)
			wndSearch:FindChild("EditBox"):SetText(k)
		end
	end
end

---------------------------------------------------------------------------------------------------
-- SearchBox Functions
---------------------------------------------------------------------------------------------------
function CircleShortcuts:OnEditBoxChanged( wndHandler, wndControl, strText )
	if not wndControl:GetName() == "EditBox" then return end
	
	if wndControl:GetData() ~= "" then
		CS_Inst.tChannels[wndControl:GetData()] = nil
	end

	wndControl:SetData(strText)
	if strText == "" then return end
	
	wndControl:SetText(string.lower(strText))
	strText = string.lower(strText)
	
	local tData = wndControl:GetParent():GetData()
	
	if not CS_Inst.tChannels[strText] then CS_Inst.tChannels[strText] = {} end
	CS_Inst.tChannels[strText].sId = tData:GetChannel():GetUniqueId()
	CS_Inst.tChannels[strText].sCommand = tData:GetChannel():GetCommand()
end

-----------------------------------------------------------------------------------------------
-- OnSave / OnRestore
-----------------------------------------------------------------------------------------------
function CircleShortcuts:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end

	local tData
	tData = self:deepcopy(tData, self.tChannels)

	return tData
end

function CircleShortcuts:OnRestore(eType, tData)
	if tData == nil then return end
	
	self.tChannels = self:deepcopy(self.tChannels, tData)
end


function CircleShortcuts:UpdateCommands()
	for k, tChannel in pairs(self.tChannels) do
		local bFound = false
		
		for _, channel in pairs(ChatSystemLib.GetChannels()) do
			if tChannel.sId == channel:GetUniqueId() then
				tChannel.sCommand = channel:GetCommand()
				bFound = true
			end
		end
		
		if not bFound then
			self.tChannels[k] = nil
		end
	end
end

function CircleShortcuts:deepcopy(tNew, tOld)
	if tOld == nil then return end
	if tNew == nil then
		tNew = {}
	end
	
	for k, v in pairs(tOld) do
		if type(v) == "table" then
			tNew[k] = self:deepcopy(tNew[k], v)
		else
			tNew[k] = v
		end
	end
	return tNew
end

-----------------------------------------------------------------------------------------------
-- CircleShortcuts Instance
-----------------------------------------------------------------------------------------------
CS_Inst = CircleShortcuts:new()
CS_Inst:Init()
